function [S,H] = getLinearSpectra_withTH(T, z, ag, Dtg)
% [S,H] = getLinearSpectra_withTH(T, z, ag, Dtg)
%
% Computes the time history response and spectral response of an elastic
% SDOF oscillator.
%
% Author: Shrey Shahi, www.github.com/shreyshahi/LinearNewmark
% Last Revised: September 15, 2012
% Reference: "Dynamics of Structures" (1995) by A.K. Chopra 
% Dependencies: linearNewmark_TH.c (see README.txt file)
%
% INPUT
% T     = period of SDOF
% z     = Damping ratio (z = 0.05 for 5% damping)  
% ag    = acceleration time history of input ground-motion  
% Dtg   = time interval at which the input acceleration is recorded  
%
% OUTPUT
% S     = spectral responses of SDOF
%   S.d = Spectral displacement  
%   S.v = pseudo spectral velocity [S.d * ( 2 * pi / T )]  
%   S.a = pseudo spectral acceleration [S.d * ( 2 * pi / T ) ^ 2]  
%
% H     = time history responses of SDOF
%   H.d = Time history of the displacement response of the oscillator  
%   H.v = Time history of the velocity response of the oscillator  
%   H.a = Time history of the acceleration response of the oscillator  
%
%--------------------------------------------------------------------------
%
% Provided as e-supplement to:
% Burks, L.S. and J.W. Baker (2013), "Validation of ground motion 
%   simulations through simple proxies for the response of engineered 
%   systems," Bulletin of the Seismological Society of America.
%
%--------------------------------------------------------------------------

% Interpolate p=-ag*m (linearly)
% ------------------------------
minT = min(T);
Dt = min([minT/10 Dtg 0.55*minT]);
tg = [ 0:(length(ag)-1) ]' * Dtg;
t = [ 0:Dt:tg(end) ]';
m=1;
p = interp1( tg, -ag*m, t );
Gamma = 1/2;
Beta = 1/6; % for linear acceleration
% Beta = 1/4; % for avg acceleration

[sd,d,v,a] = linearNewmark_TH(T,z,p,Gamma,Beta,Dt);
S.d = sd';
S.v = S.d .* (2*3.1415./T);
S.a = S.d .* (2*3.1415./T).^2;

H.d = interp1( t, d', tg );
H.v = interp1( t, v', tg );
H.a = interp1( t, a', tg );