function [RotD50, RotD100] = fn_SaRotDnn(accel1, accel2, dt, T, xi, plotFlag, axisLabels)
% [RotD50, RotD100] = fn_SaRotDnn(accel1, accel2, dt, T, xi, plotFlag, axisLabels)
% 
% Computes the spectral acceleration in the maximum direction
% and the median direction for a multi-component ground motion, and plots
% the displacement trace of an elastic ocsillator.
%
% Author: Shrey Shahi
% Last Revised: December 4, 2012
% Dependencies: getLinearSpectra_withTH.m by Shrey Shahi
%
% INPUT
% accel1        = ground acceleration in direction 1 (units of g)
% accel2        = ground acceleration in direction 2 (orthogonal to dir 1)
% dt            = time step for acceleration
% T             = period of oscillator, must be scalar
% xi            = damping ratio
% plotFlag      = optional parameter indicating whether or not to plot the
%                 direction 1 vs. direction 2 displacement time history
%               = 0 for no plot
%               = 1 to plot direction 1 vs. direction 2 disp time history
%               = 2 to plot normalized dir 1 vs. dir 2 disp time history AND
%                 normalized Sa/SaRotD100
%               = 3 to plot direction 1 vs. direction 2 disp time history with
%                 RotD100 and RotD50 labeled on plot in polar coordinates
% axisLabels    = optional parameter, cell array containing string to
%                 label axes of displacement trace plot
%
% OUPUT
% RotD50        = response spectra in the median direction, computed independently
%                 for each period from psuedo SA (units of g)
% RotD100       = response spectra in the maximum direction, computed 
%                 independently for each period  from psuedo SA (units of g)
%
%--------------------------------------------------------------------------
%
% Provided as e-supplement to:
% Burks, L.S. and J.W. Baker (2013), "Validation of ground motion 
%   simulations through simple proxies for the response of engineered 
%   systems," Bulletin of the Seismological Society of America.
%
%--------------------------------------------------------------------------

if nargin == 5
    plotFlag = 0;
    axisLabels = {'', ''};
elseif nargin == 6
    axisLabels ={'displacement in direction 1 (cm)', 'displacement in direction 2 (cm)'};
end

g = 981; % cm/s^2

accel1 = accel1 .* g;
accel2 = accel2 .* g;

[S1, H1] = getLinearSpectra_withTH(T, xi, accel1, dt);
[S2, H2] = getLinearSpectra_withTH(T, xi, accel2, dt);
Sa1 = S1.a / g;
Sa2 = S2.a / g;
Sa_abs1 = max(abs(H1.a)) / g;
Sa_abs2 = max(abs(H2.a)) / g;

% get time of Sa
time = dt:dt:length(accel1)*dt;
[~,itime1] = max(abs(H1.a));
time1 = time(itime1);
[~,itime2] = max(abs(H2.a));
time2 = time(itime2);

% compute directional spectra
disp1 = H1.d;
disp2 = H2.d;
Sa = zeros(180,1);
for i = 1:length(Sa) % increments of 1 degree
    theta = i * pi/180;
    u_directional = disp1.*cos(theta) + disp2.*sin(theta);
    Sa(i) = max(abs(u_directional)) * (2*pi/T)^2;
end
RotD50 = median(Sa);
[~,theta50] = min(abs(Sa - RotD50));
RotD50 = RotD50 / g;

[RotD100, theta100] = max(Sa);
RotD100 = RotD100 / g;

% plot oscillaor response
vector_D = abs( sqrt( (H1.d).^2 + (H2.d).^2 )); % vector displacement response amplitude
[~, index] = sort(vector_D(~isnan(vector_D)), 'descend');
if isempty(index)
    warning('no RotD100 was found');
    return
end
index_max = index(1);
x_max = H1.d(index_max,1); % x-direction response at time of max
y_max = H2.d(index_max,1); % y-direction response at time of max
if plotFlag==1 % direction 1 vs. direction 2 displacement time history
    plot(H1.d, H2.d) % in cm
    hold on
    lim_x = get(gca, 'xlim');
    lim_y = get(gca, 'ylim');
    max_lim = max(abs([lim_x lim_y]));
    axis(max_lim * [-1 1 -1 1]);
    plot([0 0], max_lim*[-1 1], '-k');
    plot(max_lim*[-1 1], [0 0], '-k');
    plot(x_max, y_max, 'ok', 'linewidth', 2);
    plot(x_max*[1 -1], y_max*[1 -1], '-k', 'linewidth', 2);
    axis square
    hx = xlabel(axisLabels{1});
    hy = ylabel(axisLabels{2});
    FormatFigure    
end
if plotFlag==2 % normalized dir 1 vs. dir 2 AND Sa/SaRotD100    
    Sa_dir = zeros(360,1);
    theta = zeros(360,1);
    for i = 1:360 % increments of 1 degree
        theta(i) = i * pi/180;
        u_directional = disp1.*cos(theta(i)) + disp2.*sin(theta(i));
        Sa_dir(i) = max(abs(u_directional)) * (2*pi/T)^2 / g;
    end
    
    % plot in polar coordinates
    hSa = polar(theta, Sa_dir./max(Sa_dir));
    hold on
    [theta2, rho] = cart2pol(disp1, disp2);
    hdisp = polar(theta2, rho./max(vector_D));
    set(hSa, 'color', 'k', 'linewidth', 2, 'linestyle', '--');
    set(hdisp, 'color', 'k', 'linewidth', 1);
    %legend([hSa hdisp], {'Sa/Sa_{RotD100}', 'Displacement/Max Displacement'}, 'location', 'southoutside');
end
if plotFlag==3    
    % find RotD100 and RotD50
    [thetaRotD100, rhoRotD100] = cart2pol(x_max, y_max);
    % plot in polar coordinates
    [theta, rho] = cart2pol(disp1, disp2);
    hdisp = polar(theta, rho);
    hold on
    hRotD100 = polar(thetaRotD100, rhoRotD100, 'ok');
    set(hRotD100, 'markersize', 10, 'linewidth', 2);
    hline = polar([thetaRotD100, thetaRotD100], [-rhoRotD100, rhoRotD100], '-k');
    set(hline, 'linewidth', 2);
end

