function [medTp, sigma_lnTp] = fn_pulsePeriod(M, method)
% [medTp, sigma_lnTp] = fn_pulsePeriod(M, method)
% 
% Returns the median of Tp and standard deviation of ln(Tp) computed 
% according to the given method
%
% Author: Lynne Burks
% Last Revised: September 6, 2013
% References: (1) Bray, J. D. and A. Rodriguez-Marek (2004). 
%   "Characterization of forward-directivity ground motions in the 
%   near-fault region," Soil Dynamics and Earthquake Engineering, 24(11), 
%   815?828.
%   (2) Shahi, S. K. and J. W. Baker (2011). "An empirically calibrated 
%   framework for including the effects of near-fault directivity in 
%   probabilistic seismic hazard analysis," Bulletin of the Seismological 
%   Society of America, 101(2), 742?755.
%
% INPUT
% M             = earthquake magnitude, can be a list
% method        = 'SB2011' for Shahi & Baker (2011)
%               = 'BRM2004' for Bray & Rodriguez-Marek (2004)
%
% OUTPUT
% medTp         = median of Tp
% sigma_lnTp    = standard deviation of ln(Tp)
%
%--------------------------------------------------------------------------
%
% Provided as e-supplement to:
% Burks, L.S. and J.W. Baker (2013), "Validation of ground motion 
%   simulations through simple proxies for the response of engineered 
%   systems," Bulletin of the Seismological Society of America.
%
%--------------------------------------------------------------------------

if strcmp(method, 'SB2011')
    medTp = exp(-5.73 + 0.99 .* M);
    sigma_lnTp = 0.56;    
elseif strcmp(method, 'BRM2004')
    medTp = exp(-5.6 + 0.93 .* M);
    sigma_lnTp = 0.58;    
else
    error(['invalid method: ' method]);
end



